import os
from typing import Tuple
import cv2
from pathlib import Path
from moviepy.editor import *
import glob

sys.path.append('./')
from utils.draw_on_video import write_text_video

extensions = ('.mp4', '.kmv', '.avi', 'mov')

def cut_video(video_file_in: str = 'samples/match.mp4', video_file_out: str = 'samples/match_clip.mp4', interval: Tuple = (0, 1)) -> None:
    """cut video during a given time interval"""
    
    myclip = VideoFileClip(video_file_in)
    myclip2 = myclip.subclip(interval[0], interval[1])
    myclip2.write_videofile(video_file_out)

    return

def crop_video(video_file_in: str = 'samples/match.mp4', video_file_out: str = 'samples/match_crop.mp4', region: Tuple = (10, 450+10, 360+10, 10)) -> None:
    top, right, bottom, left = region
    input_video = cv2.VideoCapture(video_file_in)

    fourcc = cv2.VideoWriter_fourcc(*'mp4v')
    output_movie = cv2.VideoWriter(video_file_out, fourcc, 30, (450, 360))
    while True:
        ret, frame = input_video.read()
        if not ret:
            break
        crop_img = frame[top:bottom, left:right, :]
        output_movie.write(crop_img)
    output_movie.release()

    return

def get_video_frame(video_file_in: str = 'samples/match.mp4', frame_file_out: str = 'samples/match.jpg') -> None:

    vidcap = cv2.VideoCapture(video_file_in)
    success, image = vidcap.read()

    if success:        
        write_text_video(image, "SINGLE FRAME: " + video_file_in)
        cv2.imwrite(frame_file_out, image)
        success, image = vidcap.read()
    
    return

def get_n_video_frames(video_file_in: str = 'samples/match.mp4', frame_dir_out: str = 'samples/img/', nb = 10) -> None:

    if not os.path.isdir(frame_dir_out): os.mkdir(frame_dir_out)

    vidcap = cv2.VideoCapture(video_file_in)
    success, image = vidcap.read()
    i = 0
    while success and i < nb:
        frame_name = str(i) + '.jpg'
        frame_path = os.path.join(frame_dir_out, frame_name)
        write_text_video(image, f"N FRAME: {i}")
        cv2.imwrite(frame_path, image)
        success, image = vidcap.read()
        i += 1
    
    return


def generate_video_from_frames(frame_dir_in: str = 'samples/img/', video_file_out: str = 'samples/match_gen.mp4', fps: int = 10) -> None:

    fourcc = cv2.VideoWriter_fourcc(*'mp4v')
    #fps = 10
    frameSize = (1500, 1500)

    fg = glob.glob(os.path.join(frame_dir_in, '*.jpg'))

    if fg: # If glob.glob returned list of matches
        filename = fg[0]
        image = cv2.imread(filename)
        height, width, layers = image.shape
        frameSize = (width,height)

    out = cv2.VideoWriter(video_file_out, fourcc, fps, frameSize)

    frames = os.listdir(frame_dir_in)
    frames.sort()
    frames.sort(key=len)
    # for filename in glob.glob(os.path.join(frame_dir_in, '*.jpg')):
    for filename in frames:
        filename = os.path.join(frame_dir_in, filename)
        img = cv2.imread(filename)
        # cv2.imshow('image',img)
        # cv2.waitKey(0)
        # if c & 0xFF == ord('q'):
        #    break
        out.write(img)
    
    out.release()
    # os.system(f"ffmpeg -r 25 -f image2 -i {os.path.join(frame_dir_in, '*.jpg')} -vcodec libx264 -crf 25  -pix_fmt yuv420p {video_file_out}")
    # cv2.destroyAllWindows()

    return

def background_extraction(video_file_in: str = 'samples/match.mp4', frame_dir_out: str = 'samples/bg/', nb = 10, algo = 'MOG2'):
    """This program shows how to use background subtraction methods provided by 
    OpenCV. You can process both videos and images.')"""

    if not os.path.isdir(frame_dir_out): os.mkdir(frame_dir_out)

    if algo == 'MOG2':
        backSub = cv2.createBackgroundSubtractorMOG2()
    else:
        backSub = cv2.createBackgroundSubtractorKNN()

    vidcap = cv2.VideoCapture(video_file_in)

    if not vidcap.isOpened():
        print('Unable to open: ' + video_file_in)
        exit(0)
    
    i, ret = 0, True

    while ret and i < nb:
        ret, frame = vidcap.read()
        
        fgMask = backSub.apply(frame)
        
        cv2.rectangle(frame, (10, 2), (100,20), (255,255,255), -1)
        cv2.putText(frame, str(vidcap.get(cv2.CAP_PROP_POS_FRAMES)), (15, 15),
                cv2.FONT_HERSHEY_SIMPLEX, 0.5 , (0,0,0))
        
        # cv2.imshow('Frame', frame)
        # cv2.imshow('FG Mask', fgMask)
        cv2.imwrite(os.path.join(os.getcwd(), frame_dir_out, f"{i}.jpg"), fgMask)

        i += 1

        # keyboard = cv2.waitKey(30)
        # if keyboard == 'q' or keyboard == 27:
        #    break

    return

if __name__ == "__main__":

    assert get_n_video_frames() is None
    cut_video()
    crop_video()
    get_video_frame()
    background_extraction()
    generate_video_from_frames()